import { Player, GameMode, EntityDamageCause, world, system, ItemStack, ItemLockMode } from "@minecraft/server";
import { Vector } from './Vector';
export const bullet_list = {
    "1": {
        damage: 18,
        sound_hit: "l4d2.mk18.hit",
        sound_block: "l4d2.mk18.block"
    },
};
export const gun_weapon_list = {
    "xmguns:warhammer40k_boltgun": {
        mode: 1,
        firing_rate: 3,
        reload_time: 20,
        sound_fire: "xmguns.warhammer40k.boltgun.fire"
    },
};

const item_on_reload = new ItemStack("xmguns:warhammer40k_on_reload");
item_on_reload.lockMode = ItemLockMode.slot;
item_on_reload.keepOnDeath = true;
const item_on_fire = new ItemStack("xmguns:warhammer40k_on_fire");
item_on_fire.lockMode = ItemLockMode.slot;
item_on_fire.keepOnDeath = true;
const item_off_fire = new ItemStack("xmguns:warhammer40k_off_fire");
item_off_fire.lockMode = ItemLockMode.slot;
item_off_fire.keepOnDeath = true;
let currentSlot = 10;


// world.afterEvents.entityHitBlock.subscribe((xm) => {
//     const player = xm.damagingEntity;
//     if (!(player instanceof Player))
//         return;
//     const inv = player.getComponent("inventory").container;
//     const item = inv.getItem(player.selectedSlotIndex);
//     const item_id = item?.typeId;
//     const gun_weapon_id = gun_weapon_list[item_id];
//     if (!gun_weapon_id)
//         return;
// });

world.afterEvents.itemStopUse.subscribe((xm) => {
    const { itemStack, source } = xm;
    const player = source;
    if (!(player instanceof Player))
        return;
    const item = itemStack;
    const item_id = item.typeId;
    const gun_weapon_id = gun_weapon_list[item_id];
    if (!gun_weapon_id)
        return;
    const durability = item.getComponent('minecraft:durability');
    if (durability.damage == durability.maxDurability)
        return;
    const inv = player.getComponent('minecraft:inventory').container;
    inv.setItem(17, item_off_fire);
    player.removeTag("l4d2:gun_fire_on");
    player.removeTag("l4d2:gun_shoot");
    // player.sendMessage("停火");
});


world.afterEvents.itemStartUse.subscribe((xm) => {
    const { source, source: { dimension } } = xm;
    const player = source;
    if (!(player instanceof Player)) return;


    const item = xm.itemStack;
    const item_id = item?.typeId;
    const gun_weapon_id = gun_weapon_list[item_id];
    if (!gun_weapon_id) return;



    const durability = item.getComponent('minecraft:durability');
    if (durability.damage == durability.maxDurability) {
        if (player.hasTag("l4d2:item_on_reload")) return;

        const reload = system.runInterval(() => {
            const inv = player.getComponent('minecraft:inventory').container;
            inv.setItem(17, item_on_reload);

            if (currentSlot == 10) {
                currentSlot = player.selectedSlotIndex;
            }
            system.runTimeout(() => {
                inv.setItem(17, item_off_fire);
                if (currentSlot != 10) {
                    durability.damage = 0;
                    inv?.setItem(currentSlot, item);
                    // world.sendMessage(`${durability.damage}`);
                    currentSlot = 10;
                }
            }, gun_weapon_id.reload_time);

            player.addTag("l4d2:item_on_reload");
            system.clearRun(reload);
            return;
        });
    }
    else {
        if (player.hasTag("l4d2:gun_fire_on")) return;

        let wait_shoot_time = 0;

        const shoot = system.runInterval(() => {
            const location = player.location;
            const health = player.getComponent("health");
            const fire_on = player.hasTag("l4d2:gun_fire_on");
            const is_fire = player.hasTag("l4d2:gun_is_fire");
            const durability = item.getComponent('minecraft:durability');
            const inv = player.getComponent('minecraft:inventory').container;
            if (currentSlot == 10) {
                currentSlot = player.selectedSlotIndex;
            }
            if (health.currentValue < 1) {
                currentSlot = 10;
                system.clearRun(shoot);
                return;
            }
            if ((!fire_on && !is_fire)) {
                if (currentSlot != 10) {
                    system.clearRun(shoot);
                    currentSlot = 10;
                    return;
                }
            }
            if ((!fire_on && is_fire) || durability.damage == durability.maxDurability) {
                if (currentSlot != 10) {
                    player.removeTag("l4d2:gun_is_fire");
                    inv?.setItem(currentSlot, item);
                    // world.sendMessage(`${currentSlot}格子${durability.damage}`);
                    system.clearRun(shoot);
                    currentSlot = 10;
                    return;
                }
            }


            if (player.hasTag("l4d2:gun_shoot")) return;

            system.runTimeout(() => {
                if (!player.hasTag("l4d2:gun_fire_on")) return;
                inv.setItem(17, item_on_fire);

                const viewDirection = source.getViewDirection();
                const rightVector = Vector.cross(viewDirection, { x: 0, y: 1, z: 0 });
                const rightVectorNormal = rightVector.normalized();
                const rightVectorRandom = Vector.multiply(rightVectorNormal, 0.4);
                const directionForRaycast = Vector.add(viewDirection, rightVectorRandom);
                const bullet = dimension.spawnEntity(`xmguns:warhammer40k_bullet` + `<${item_id}>`, Vector.add(source.getHeadLocation(), directionForRaycast));
                const p = bullet.getComponent('projectile');
                p.owner = player;
                p.shoot(Vector.add(Vector.multiply(source.getViewDirection(), 6), { x: 0, y: 0, z: 0 }));
                world.playSound(gun_weapon_id.sound_fire, location, { volume: 1 });
                durability.damage += 1;
                // world.sendMessage(`${durability.damage}`);
                system.runTimeout(() => {
                    inv.setItem(17, item_off_fire);
                }, 1);
                player.addTag("l4d2:gun_is_fire");
                player.removeTag("l4d2:gun_shoot");
            }, gun_weapon_id.firing_rate);
            player.addTag("l4d2:gun_shoot");


            wait_shoot_time += 0.05;
        });
        player.addTag("l4d2:gun_fire_on");
        // player.sendMessage("开火");
    }
});
world.afterEvents.entityDie.subscribe((xm) => {
    const player = xm.deadEntity;
    if (!(player instanceof Player))
        return;
    player.removeTag("l4d2:gun_fire_on");
    player.removeTag("l4d2:gun_shoot");
    // player.sendMessage("停火");
});
world.afterEvents.projectileHitEntity.subscribe((xm) => {
    const bullet = xm.projectile;
    if (bullet.typeId != "xmguns:warhammer40k_bullet")
        return;
    if (bullet.typeId == "xmguns:warhammer40k_bullet") {
        const entity_hit = xm.getEntityHit();
        const hitEntity = entity_hit.entity;
        if (entity_hit instanceof Player) {
            const hit_creative = hitEntity.matches({ gameMode: GameMode.creative }) || hitEntity.matches({ gameMode: GameMode.spectator });
            if (hit_creative == true)
                return;
        }
        const bullet_id = bullet.getComponent("minecraft:skin_id")?.value ?? 0;
        const bullet_def = bullet_list[bullet_id];
        if (!bullet_def)
            return;
        hitEntity.applyDamage(bullet_def.damage, {
            cause: EntityDamageCause.suicide,
            damagingProjectile: bullet,
        });
        // const location = xm.location;
        // const soundOption = {
        //     volume: 1,
        //     pitch: 1,
        // };
        // world.playSound(bullet_def.sound_hit, location, soundOption);
    }
});



const overworld = world.getDimension("overworld");
const nether = world.getDimension("nether");
const the_end = world.getDimension("the_end");
const l4d2_on_damage_timer = new Map();
system.runInterval(() => {
    const entities = [...overworld.getEntities(), ...nether.getEntities(), ...the_end.getEntities()];
    entities.forEach((entity) => {
        const l4d2_on_damage = l4d2_damage(entity);
        const damage_timer = l4d2_on_damage_timer.get(entity.id);
        if (!l4d2_on_damage && damage_timer)
            system.clearRun(damage_timer);
        if (l4d2_on_damage && !damage_timer) {
            l4d2_on_damage_timer.set(entity.id, system.runTimeout(() => {
                entity.removeTag('l4d2:item_on_reload');
                l4d2_on_damage_timer.delete(entity.id);
            }, 20));
        }
    });
});
function l4d2_damage(entity) {
    return entity.hasTag("l4d2:item_on_reload");
}